@@ -30,7 +30,7 @@
 #'     others. When TRUE, adds an AUC column to the prediction results. (default: TRUE)
 #' @param plot_AUC The logic indicates whether to draw an AUC curve for the predicted cell
 #'     type. When TRUE, add an AUC_plot to result. (default: TRUE)
-#' @param AUC_correction Logical value controlling AUC-based correction. (default = TRUE)
+#' @param AUC_correction Logical value controlling AUC-based correction. (default = FALSE)
 #'     When set to TRUE:
 #'     1.Computes AUC values for candidate cell types. (probability > threshold)
 #'     2.Selects the cell type with the highest AUC as the final predicted type.
@@ -53,8 +53,9 @@
 #'       \item AUC: Area Under the Curve value (when compute_AUC = TRUE)
 #'       \item Alternative_cell_types: Semi-colon separated alternative cell types
 #'     }
-#'   \item Heatmap_plot: Heatmap visualization of probability matrix
-#'   \item AUC_plot: AUC visualization of Predicted cell type
+#'   \item Heatmap_plot: Heatmap visualization of probability matrix (pheatmap object).
+#'         Can be displayed using \code{print()} or \code{plot()}
+#'   \item AUC_plot: AUC visualization of Predicted cell type (ggplot object)
 #'   \item AUC_list: The resulting list of AUC values calculated for genes in alternative cell types above the approximate threshold
 #' }
 #'
@@ -63,6 +64,7 @@
 #'
 #' @importFrom grDevices colorRampPalette
 #' @importFrom utils tail
+#' @importFrom stats runif
 #' @importFrom ggplot2 ggplot aes geom_line geom_abline scale_color_manual
 #' @importFrom ggplot2 theme_minimal labs theme element_text element_blank
 #' @importFrom ggplot2 guide_legend guides scale_x_continuous scale_y_continuous
@@ -81,7 +83,7 @@
 #'     threshold = 0.6,
 #'     compute_AUC = TRUE,
 #'     plot_AUC = TRUE,
-#'     AUC_correction = TRUE,
+#'     AUC_correction = FALSE,
 #'     colour_low = "navy",
 #'     colour_high = "firebrick3"
 #'     )
@@ -98,7 +100,7 @@ Celltype_Calculate <- function(
     threshold = 0.6,
     compute_AUC = TRUE,
     plot_AUC = TRUE,
-    AUC_correction = TRUE,
+    AUC_correction = FALSE,
     colour_low = "navy",
     colour_high = "firebrick3"
 ) {
@@ -197,6 +199,13 @@ Celltype_Calculate <- function(
   normalize_matrix <- apply(scores_matrix, 2, normalize_row)
   result_matrix <- t(normalize_matrix)
 
+  # Check if all values are the same (would cause pheatmap to fail)
+  if (length(unique(as.vector(result_matrix))) == 1) {
+    # Add small random noise to prevent identical values
+    result_matrix <- result_matrix + matrix(runif(length(result_matrix), -1e-10, 1e-10), 
+                                             nrow = nrow(result_matrix))
+  }
+
   p <- pheatmap::pheatmap(result_matrix,
                           main = "Cell annotation heatmap | SlimR",
                           color = colorRampPalette(c(colour_low, "white", colour_high))(50),
@@ -577,8 +586,12 @@ Celltype_Calculate <- function(
     return_list$AUC_plot <- auc_plot
   }
   
-  if (compute_AUC && exists("auc_list_storage") && length(auc_list_storage) > 0) {
-    return_list$AUC_list <- auc_list_storage
+  if (compute_AUC) {
+    if (exists("auc_list_storage")) {
+      return_list$AUC_list <- auc_list_storage
+    } else {
+      return_list$AUC_list <- list()
+    }
   }
 
   return(return_list)
